"""
Use this file to configure pluggable app settings and resolve defaults
with any overrides set in project settings.
"""

from django.conf import settings as project_settings
from django.utils.text import slugify

from staff.utils.importers import import_class


class Settings:
    pass


Settings.API_AUTHENTICATION_CLASS = import_class(
    getattr(
        project_settings,
        "STAFF_API_AUTHENTICATION_CLASS",
        "tokenservice.authentication.TokenAuthentication",
    )
)

Settings.API_PERMISSION_CLASS = import_class(
    getattr(
        project_settings,
        "STAFF_API_PERMISSION_CLASS",
        "rest_framework.permissions.IsAuthenticated",
    )
)


Settings.SLACK_API_TOKEN = getattr(
    project_settings, "STAFF_SLACK_API_TOKEN", None
)

Settings.SECRET_TOKEN = getattr(
    project_settings, "STAFF_SECRET_TOKEN", "SECRET"
)

Settings.AUTH_DECORATOR = getattr(
    project_settings,
    "STAFF_AUTH_DECORATOR",
    "django.contrib.admin.views.decorators.staff_member_required",
)


def default_user_image_upload_to(instance, filename):
    return "staff/{0}/{1}".format(
        slugify(f"{instance.user.first_name} {instance.user.last_name}"),
        filename,
    )


Settings.USER_IMAGE_UPLOAD_TO = getattr(
    project_settings,
    "STAFF_USER_IMAGE_UPLOAD_TO",
    default_user_image_upload_to,
)


settings = Settings



from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from rest_framework import status
from rest_framework.parsers import JSONParser
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from staff.celery import sync_slack_users
from staff.utils.auth import TokenAuthentication


class SyncUser(APIView):
    """
    Request a users profile synced from Slack.
    """

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    parser_classes = (JSONParser,)

    def post(self, request, format=None):
        pk = request.data.get("user", None)
        try:
            user = User.objects.get(pk=pk)
        except ObjectDoesNotExist:
            return Response(
                {"msg": "User not found"}, status=status.HTTP_404_NOT_FOUND
            )
        sync_slack_users.delay([user.pk])
        return Response({"msg": "Requested sync with Slack"})



from rest_framework.serializers import ModelSerializer

from staff.models import Profile


class ProfileSerializer(ModelSerializer):
    class Meta:
        model = Profile
        fields = (
            "id",
            "politico_title",
            "politico_author_page",
            "slack_api_id",
            "slack_image",
            "google_email",
            "google_display_name",
            "twitter_handle",
        )


class EditableProfileSerializer(ModelSerializer):
    class Meta:
        model = Profile
        fields = (
            "id",
            "politico_author_page",
            "google_email",
            "google_display_name",
            "twitter_handle",
        )



from django.conf import settings as project_settings
from django.contrib.auth.models import AnonymousUser
from django.utils.decorators import method_decorator
from rest_framework import authentication, exceptions

from staff.conf import settings

from .importers import import_class


def secure(view):
    """
    Authentication decorator for views.

    If DEBUG is on, we serve the view without authenticating.
    Default is 'django.contrib.auth.decorators.login_required'.
    Can also be 'django.contrib.admin.views.decorators.staff_member_required'
    or a custom decorator.
    """
    auth_decorator = import_class(settings.AUTH_DECORATOR)
    return (
        view
        if project_settings.DEBUG
        else method_decorator(auth_decorator, name="dispatch")(view)
    )


class TokenAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        # Don't enforce if DEBUG
        # if settings.DEBUG:
        #     return (AnonymousUser, None)
        try:
            # Per DRF token auth, token is prefixed by string
            # literal "Token" plus whitespace, e.g., "Token <AUTHTOKEN>"
            token = request.META.get("HTTP_AUTHORIZATION").split()[1]
        except Exception:
            raise exceptions.AuthenticationFailed(
                "No token or incorrect token format"
            )

        if token == settings.SECRET_TOKEN:
            return (AnonymousUser, None)
        raise exceptions.AuthenticationFailed("Unauthorized")




from django.urls import include, path
from rest_framework import routers

from .views import EditProfile, PatchProfile, SyncUser
from .viewsets import StafferViewSet

router = routers.DefaultRouter()

router.register(r"staffer", StafferViewSet, basename="staffer")

urlpatterns = [
    path("api/", include(router.urls)),
    path("edit-profile/", EditProfile.as_view(), name="staff-edit-profile"),
    path("sync-profile/", SyncUser.as_view(), name="staff-sync-profile"),
    path("patch-profile/", PatchProfile.as_view(), name="staff-patch-profile"),
]



import uuid

import requests
from django.contrib.auth.models import User
from django.core.files.base import ContentFile
from django.core.management.base import BaseCommand
from tqdm import tqdm

from slacker import Slacker
from staff.conf import settings
from staff.models import Profile


class Command(BaseCommand):
    help = "Sync users from Slack"

    def handle(self, *args, **options):
        SLACK = Slacker(settings.SLACK_API_TOKEN)
        slack_users = SLACK.users.list().body["members"]
        for slack_user in tqdm(slack_users, desc="Users"):
            id = slack_user["id"]
            slack_profile = slack_user["profile"]

            real_name = slack_profile.get("real_name", None)
            display_name = slack_profile.get("display_name", None)
            try:
                first_name, last_name = real_name.split(" ", 1)
            except ValueError:
                try:
                    first_name, last_name = display_name.split(" ", 1)
                except ValueError:
                    first_name = real_name or display_name or ""
                    last_name = ""

            email = slack_profile.get("email", None)

            if not email or email[-13:] != "@politico.com":
                continue

            user, created = User.objects.update_or_create(
                username=email,
                defaults={
                    "email": email,
                    "first_name": first_name,
                    "last_name": last_name,
                    "is_staff": True,
                    "is_active": slack_user["deleted"] is False,
                },
            )

            profile, created = Profile.objects.update_or_create(
                user=user,
                defaults={
                    "slack_api_id": id,
                    "politico_title": slack_profile.get("title", "Staff"),
                },
            )

            if (
                slack_profile.get("image_192", False)
                and not profile.slack_image
            ):
                r = requests.get(slack_profile.get("image_192"), stream=True)
                img = r.raw.read()
                profile.slack_image.save(
                    "slack-profile-{}.jpg".format(uuid.uuid4().hex[:10]),
                    ContentFile(img),
                    save=True,
                )
                profile.save()



from django.contrib.auth.models import User
from rest_framework.viewsets import ReadOnlyModelViewSet

from staff.conf import settings
from staff.serializers import UserSerializer


class StafferViewSet(ReadOnlyModelViewSet):
    serializer_class = UserSerializer
    authentication_classes = (settings.API_AUTHENTICATION_CLASS,)
    permission_classes = (settings.API_PERMISSION_CLASS,)
    pagination_class = None

    def get_queryset(self):
        queryset = User.objects.filter(is_staff=True)
        is_active = self.request.query_params.get("active", None) in [
            "true",
            "1",
        ]
        if is_active:
            queryset = queryset.filter(is_active=True)
        return queryset



from django.contrib.auth.models import User
from django.db import models

from staff.conf import settings


class Profile(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)

    # politico.com
    politico_title = models.CharField(
        max_length=100, blank=True, null=True, verbose_name="Title"
    )
    politico_author_page = models.URLField(
        blank=True, null=True, verbose_name="Author page"
    )

    # Slack
    slack_api_id = models.CharField(
        max_length=50, blank=True, null=True, verbose_name="ID"
    )
    slack_image = models.ImageField(
        upload_to=settings.USER_IMAGE_UPLOAD_TO,
        blank=True,
        null=True,
        verbose_name="Profile pic",
    )

    # Google
    google_email = models.EmailField(
        blank=True, null=True, verbose_name="Gmail"
    )
    google_display_name = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Display name"
    )

    # twitter
    twitter_handle = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Handle"
    )

    def __str__(self):
        return self.user.email



from django.contrib.auth.models import User
from django.db.models.signals import post_save
from django.dispatch import receiver

from .celery import sync_slack_users


@receiver(post_save, sender=User)
def auto_staff(sender, instance, created, **kwargs):
    """Automatically promote user to staff if email is politico.com.
    """
    if created:
        if instance.email[-13:] == "@politico.com" and not instance.is_staff:
            instance.is_staff = True
            instance.save()
            if instance.pk:
                sync_slack_users.delay([instance.pk])



from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.urls import reverse
from django.views.generic.base import TemplateView
from rest_framework import status
from rest_framework.parsers import JSONParser
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from staff.conf import settings
from staff.models import Profile
from staff.serializers import EditableProfileSerializer, UserSerializer
from staff.utils.auth import TokenAuthentication, secure


@secure
class EditProfile(TemplateView):
    template_name = "staff/edit.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        try:
            user = User.objects.get(pk=self.request.user.pk)
        except User.DoesNotExit:
            raise Http404
        context["user"] = user
        serializer = UserSerializer(user)
        context["serialized_user"] = serializer.data
        context["api"] = {
            "sync": reverse("staff-sync-profile"),
            "patch": reverse("staff-patch-profile"),
            "token": settings.SECRET_TOKEN,
        }
        return context


class PatchProfile(APIView):
    """
    Patch a profile
    """

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    parser_classes = (JSONParser,)

    def patch(self, request, format=None):
        profile_id = request.data.get("id", None)
        try:
            profile = Profile.objects.get(pk=profile_id)
        except ObjectDoesNotExist:
            return Response(
                {"msg": "Profile not found"}, status=status.HTTP_404_NOT_FOUND
            )
        serializer = EditableProfileSerializer(
            profile, data=request.data, partial=True
        )
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)



import uuid

import requests
from celery import shared_task
from django.contrib.auth.models import User
from django.core.files.base import ContentFile

from slacker import Slacker
from staff.conf import settings
from staff.models import Profile


def get_slack_user(user, slack_users):
    """Return user profile from Slack."""
    if hasattr(user, "profile"):
        if user.profile.slack_api_id:
            for slack_user in slack_users:
                if slack_user["id"] == user.profile.slack_api_id:
                    return slack_user
        return None
    else:
        for slack_user in slack_users:
            if slack_user["profile"]["email"] == user.email:
                return slack_user
        return None


@shared_task(acks_late=True)
def sync_slack_users(pks):
    SLACK = Slacker(settings.SLACK_API_TOKEN)
    slack_users = SLACK.users.list().body["members"]
    for pk in pks:
        user = User.objects.get(pk=pk)

        if not user.email or user.email[-13:] != "@politico.com":
            continue

        slack_user = get_slack_user(user, slack_users)

        if not slack_user:
            continue

        slack_profile = slack_user["profile"]

        real_name = slack_profile.get("real_name", None)
        display_name = slack_profile.get("display_name", None)
        try:
            first_name, last_name = real_name.split(" ", 1)
        except ValueError:
            try:
                first_name, last_name = display_name.split(" ", 1)
            except ValueError:
                first_name = real_name or display_name or ""
                last_name = ""

        user.first_name = first_name
        user.last_name = last_name
        user.save()

        profile, created = Profile.objects.update_or_create(
            user=user,
            defaults={
                "slack_api_id": slack_profile["id"],
                "politico_title": slack_profile.get("title", "Staff writer"),
            },
        )

        if slack_profile.get("image_192", False):
            r = requests.get(slack_profile.get("image_192"), stream=True)
            img = r.raw.read()
            profile.slack_image.save(
                "slack-profile-{}.jpg".format(uuid.uuid4().hex[:10]),
                ContentFile(img),
                save=True,
            )
            profile.save()



from django.contrib.auth.models import User
from rest_framework.serializers import ModelSerializer

from .profile import ProfileSerializer


class UserSerializer(ModelSerializer):
    profile = ProfileSerializer()

    class Meta:
        model = User
        fields = (
            "id",
            "username",
            "first_name",
            "last_name",
            "email",
            "is_staff",
            "is_active",
            "profile",
        )



# Generated by Django 2.1.5 on 2019-01-16 20:05

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import staff.conf


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('politico_title', models.CharField(blank=True, max_length=100, null=True, verbose_name='Title')),
                ('politico_author_page', models.URLField(blank=True, null=True, verbose_name='Author page')),
                ('slack_api_id', models.CharField(blank=True, max_length=50, null=True, verbose_name='ID')),
                ('slack_image', models.ImageField(blank=True, null=True, upload_to=staff.conf.default_user_image_upload_to, verbose_name='Profile pic')),
                ('google_email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='Gmail')),
                ('google_display_name', models.CharField(blank=True, max_length=250, null=True, verbose_name='Display name')),
                ('twitter_handle', models.CharField(blank=True, max_length=250, null=True, verbose_name='Handle')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.contrib.auth.models import User

from staff.models import Profile

from .celery import sync_slack_users


class ProfileInlineAdmin(admin.StackedInline):
    model = Profile
    can_delete = False
    verbose_name_plural = "user profile"
    fieldsets = (
        (None, {"fields": ("user",)}),
        ("Politico", {"fields": ("politico_title", "politico_author_page")}),
        ("Slack", {"fields": ("slack_api_id", "slack_image")}),
        ("Google", {"fields": ("google_email", "google_display_name")}),
        ("Twitter", {"fields": ("twitter_handle",)}),
    )
    readonly_fields = ("slack_api_id", "slack_image")


class UserAdmin(BaseUserAdmin):
    inlines = (ProfileInlineAdmin,)

    actions = ["sync_slack_profile"]

    def sync_slack_profile(self, request, queryset):
        sync_slack_users.delay([user.pk for user in queryset])
        self.message_user(request, "Synced with Slack")


admin.site.unregister(User)
admin.site.register(User, UserAdmin)
